<?php
/* $Id: nc_cache_full.class.php 3920 2010-08-23 11:10:01Z denis $ */

/**
 * class nc_cache_full
 * @package nc_cache_full
 * @category nc_cache
 */
class nc_cache_full extends nc_cache {

  /**
   * Constructor method
   *
   * Instantiate in getObject() static method
   * Singleton pattern
   */
  protected function __construct () {
    parent::__construct();
    // set essence
    $this->essence = "full";
    // create dir if no exist
    $this->createWritableDir($this->cache_path, $this->essence);
    // append path
    $this->cache_path.= $this->essence."/";
    // bind actions
    $this->core->event->bind( $this, array("updateCatalogue" => "dropCatalogueCache") );
    $this->core->event->bind( $this, array("dropCatalogue" => "dropCatalogueCache") );
    
    $this->core->event->bind( $this, array("updateSubdivision" => "dropSubdivisionCache") );
    $this->core->event->bind( $this, array("dropSubdivision" => "dropSubdivisionCache") );
    
    $this->core->event->bind( $this, array("updateSubClass" => "dropSubClassCache") );
    $this->core->event->bind( $this, array("dropSubClass" => "dropSubClassCache") );
    
    $this->core->event->bind( $this, array("updateClass" => "dropClassCache") );
    $this->core->event->bind( $this, array("dropClass" => "dropClassCache") );
    
    $this->core->event->bind( $this, array("updateClassTemplate" => "dropClassTemplateCache") );
    $this->core->event->bind( $this, array("dropClassTemplate" => "dropClassTemplateCache") );

    $this->core->event->bind( $this, array("updateMessage" => "dropMessageCache") );
    $this->core->event->bind( $this, array("dropMessage" => "dropMessageCache") );
    $this->core->event->bind( $this, array("checkMessage" => "dropMessageCache") );
    $this->core->event->bind( $this, array("uncheckMessage" => "dropMessageCache") );
  }
  
  /**
   * Get or instance self object
   *
   * @return self object
   */
  public static function getObject () {
    // call as static
    static $storage;
    // check inited object
    if ( !isset($storage) ) {
      // init object
      $storage = new self();
    }    
    // return object
    return is_object($storage) ? $storage : false;
  }
  
  /**
   * Add cache on disk method
   *
   * @param int Class_ID
   * @param int Message_ID
   * @param string $REQUEST_URI parameter
   * @param string caching data
   *
   * @return mixed bytes writed or false
   */
  public function add ($classID, $message, $query_string, $data, $cache_vars) {
    // don't cache in admin mode
    if ($this->admin_mode || $this->inside_admin) return false;
    // if object set in unactive status
    if (!$this->settings["Status_".$this->essence]) return false;
    // check possibility
    if ( !$this->_possibility() ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNRECOGNIZED_OBJECT_CALLING);
    }
    
    // validate
    if (
      !(is_numeric($classID) && $classID > 0) ||
      !(is_numeric($message) && $message > 0) ||
      ( $query_string && !is_string($query_string) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    // append audit info
    $this->auditUpdateInfo( array(0, 0, 0, $classID, $message), $query_string, "", "write", true);
    
    // cache info prefix for file
    $cache_prefix = "/* cache created ".time()." */\n";
    
    // check quota and clear old cache or return
    $data_size = strlen($cache_prefix.$data) + ( is_array($cache_vars) ? serialize($cache_vars) : 0 );
    if ( !$this->checkQuota($data_size) ) return false;
    
    // create class directory
    if ( $this->createWritableDir($this->cache_path, $classID) ) {
      $file_path = $this->cache_path.$classID."/";
    }
    else {
      return false;
    }
    
    // create message directory
    if ( $this->createWritableDir($file_path, $message) ) {
      $file_path.= $message."/";
    }
    else {
      return false;
    }

    $bytes_writed = 0;
    
    // cache file path
    $file_path.= md5($query_string);
    
    // variables file path
    $file_vars_path = $file_path.".vars.php";

    // set file extension and update data if need it
    if ( $new_data = $this->nocacheReplace($data) ) {
      $data = $new_data;
      $file_ext = ".php";
    }
    else {
      $file_ext = ".html.php";
    }
    
    // append variables file
    if ( !empty($cache_vars) ) {
      $bytes_writed+= $this->addVariables($file_vars_path, $cache_vars);
    }
    
    // file with extension
    $file_full_path = $file_path.$file_ext;
    
    // get current file size
    $exist_bytes = file_exists($file_full_path) ? filesize($file_full_path) : 0;
    $exist_bytes+= file_exists($file_vars_path) ? filesize($file_vars_path) : 0;
    
    // write data into the file
    if ( $bytes_writed+= @file_put_contents($file_full_path, $cache_prefix.$data) ) {
      // update cache stat
      if ($bytes_writed - $exist_bytes) $this->updateStat($bytes_writed - $exist_bytes);
      // append audit info
      $this->auditUpdateInfo( array(0, 0, 0, $classID, $message), $query_string, "", "write");
    }
    else {
      throw new Exception ( str_replace("%FILE", $file_full_path, NETCAT_MODULE_CACHE_CLASS_CANNOT_CREATE_FILE) );
    }
    
    // return result
    return $bytes_writed ? $bytes_writed : false;
  }
  
  /**
   * Read cached file from disk method
   *
   * @param int Class_ID
   * @param int Message_ID
   * @param string $query_string parameter
   * @param int cache lifetime
   *
   * @return mixed file data or -1 as false
   */  
  public function read ($classID, $message, $query_string, $lifetime = 0) {
    // if object set in unactive status
    if (!$this->settings["Status_".$this->essence]) return -1;
    // check possibility
    if ( !$this->_possibility() ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNRECOGNIZED_OBJECT_CALLING);
    }
    // validate
    if (
      !(is_numeric($classID) && $classID > 0) ||
      !(is_numeric($message) && $message > 0) ||
      ( $query_string && !is_string($query_string) ) ||
      !is_numeric($lifetime)
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    // append audit info
    $this->auditUpdateInfo( array(0, 0, 0, $classID, $message), $query_string, "", "read", true);
    
    // startup values
    $result = false;
    $cache_vars = array();
    
    // cache file path
    $file_path = $this->cache_path.$classID."/".$message."/".md5($query_string);
    // check cached file
    switch (true) {
      case file_exists($file_path.".html.php"):
        $file_exist_path = $file_path.".html.php";
        $content = file_get_contents($file_exist_path);
        $content_eval = false;
      break;
      case file_exists($file_path.".php"):
        $file_exist_path = $file_path.".php";
        $content_eval = true;
        $content = file_get_contents($file_exist_path);
      break;
    }
    
    // read variables file
    $cache_vars = $this->readVariables($file_path.".vars.php");
    
        
    // no cache data and cache info in the file
    if (!$content) return -1;
    // check cache expiration
    if ( $this->checkExpire($content, $lifetime) ) {
      $result = array($content, $content_eval, $cache_vars);
      // append audit info
      $this->auditUpdateInfo( array(0, 0, 0, $classID, $message), $query_string, "", "read");
    }
    else {
      // delete file and update stat
      if ( is_writable( dirname($file_exist_path) ) ) {
        // data to delete size
        $unlink_data_size = -filesize($file_exist_path);
        // delete file
        if ( unlink($file_exist_path) ) {
          // delete variables file
          $vars_bytes_droped = $this->dropVariables ($file_path.".vars.php");
          // update main stat
          $this->updateStat($unlink_data_size - $vars_bytes_droped);
        }
        // unset
        unset ($unlink_data_size);
      }
    }
    // return false
    return $result ? $result : -1;
  }
  
  
  /**
   * Add cache variables file on disk
   *
   * @param string file path
   * @param array caching data
   *
   * @return mixed bytes writed or false
   */
  protected function addVariables ($file_path, $cache_vars) {
  
    // validate
    if (
      !is_array($cache_vars)
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    $bytes_writed = 0;
    
    if ( !empty($cache_vars) ) {
      $bytes_writed = @file_put_contents( $file_path, serialize($cache_vars) );
    }
    
    return $bytes_writed;
  }
  
  
  /**
   * Read cached variables file from disk
   *
   * @param string file path
   *
   * @return array file data
   */  
  protected function readVariables ($file_path) {
  
    $result = array();
    
    if ( file_exists($file_path) ) {
      $cache_vars = @file_get_contents($file_path);
    }
    
    if ($cache_vars) $result = unserialize($cache_vars);
    
    return $result;
  }
  
  
  /**
   * Delete cached variables file from disk
   *
   * @param string file path
   *
   * @return int bytes droped
   */
  protected function dropVariables ($file_path) {
    
    $bytes_droped = 0;
    
    if ( file_exists($file_path) ) {
      // get file size
      $bytes_droped = filesize($file_path);
      // delete file
      if ( unlink($file_path) ) {
        return $bytes_droped;
      }
    }
    
    return $bytes_droped;
  }
  
  
  /**
   * Check parent function by backtrace
   *
   * @return true or false
   */
  private function _possibility () {
    // this function must be called only from "/netcat/full.php" file
    $debug_backtrace = debug_backtrace();
    // get file from calling this method
    $deb_value = $debug_backtrace[1];
    if ( str_replace( array("/", "\\"), "/", $deb_value['file']) == str_replace( array("/", "\\"), "/", $this->root_folder."full.php") ) {
      return true;
    }
    return false;
  }
  
  /**
   * Delete cache with setted efficiency (low efficiency == 1)
   *
   * @param int efficiency (1 - low, 2- middle, 3 - good)
   *
   * @return int total deleted bytes
   */
  protected function dropEfficiencyCache ($efficiency) {
    // get cache with efficiency == 1 (low)
    $clear_data = $this->getEfficiencyCache($efficiency);
    // return if empty
    if ( empty($clear_data) ) return false;
    // total deleted bytes
    $total_bytes = 0;
    // try delete cache
    foreach ($clear_data AS $value) {
      $total_bytes+= $this->dropMessageCache(0, 0, 0, $value['Class_ID'], $value['Message_ID']);
    }
    // return total deleted bytes
    return $total_bytes;
  }

  /**
   * Delete catalogue cache dir from disk method
   *
   * @param mixed Catalogue_ID
   *
   * @return int total deleted bytes
   */   
  public function dropCatalogueCache ($catalogue) {
    // validate
    if (
      !( ( is_numeric($catalogue) && $catalogue > 0 ) || is_array($catalogue) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    $catalogue = (array)$catalogue;
    if ( empty($catalogue) ) return 0;
    
    // get all catalogues classes
    $class = $this->db->get_col("SELECT DISTINCT `Class_ID` FROM `Sub_Class`
      WHERE `Catalogue_ID` IN (".join(", ", $catalogue).")");

    // return total deleted bytes
    return !empty($class) ? $this->dropClassCache($class) : 0;
  }

  /**
   * Delete subdivision cache dir from disk method
   *
   * @param int Catalogue_ID
   * @param mixed Subdivision_ID
   *
   * @return int total deleted bytes
   */   
  public function dropSubdivisionCache ($catalogue, $sub) {
    // validate
    if (
      !( is_numeric($catalogue) && $catalogue > 0 ) ||
      !( ( is_numeric($sub) && $sub > 0 ) || is_array($sub) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    $sub = (array)$sub;
    if ( empty($sub) ) return 0;
    
    // get all catalogues classes
    $class = $this->db->get_col("SELECT DISTINCT `Class_ID` FROM `Sub_Class`
      WHERE `Catalogue_ID` = '".$catalogue."'
      AND `Subdivision_ID` IN (".join(", ", $sub).")");

    // return total deleted bytes
    return !empty($class) ? $this->dropClassCache($class) : 0;
  }

  /**
   * Delete subclass cache dir from disk method
   *
   * @param int Catalogue_ID
   * @param int Subdivision_ID
   * @param mixed Sub_Class_ID
   *
   * @return int total deleted bytes
   */  
  public function dropSubClassCache ($catalogue, $sub, $cc) {
    // validate
    if (
      !( is_numeric($catalogue) && $catalogue > 0 ) ||
      !( is_numeric($sub) && $sub > 0 ) ||
      !( ( is_numeric($cc) && $cc > 0 ) || is_array($cc) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    // get all catalogues classes
    $class = $this->db->get_col("SELECT DISTINCT `Class_ID` FROM `Sub_Class`
      WHERE `Catalogue_ID` = '".$catalogue."'
      AND `Subdivision_ID` = '".$sub."'
      AND `Sub_Class_ID` IN (".join(", ", (array)$cc).")");

    // return total deleted bytes
    return !empty($class) ? $this->dropClassCache($class) : 0;
  }
  
  /**
   * Delete component cache dir from disk method
   *
   * @param int Class_ID
   *
   * @return int total deleted bytes
   */
  public function dropClassCache ($class) {
    // validate
    if (
      !( (is_numeric($class) && $class > 0 ) || is_array($class) )
    ) {
      return false;
    }
    // total deleted bytes
    $total_bytes = 0;
    foreach ( (array)$class AS $classID ) {
      // cache file path
      $path = $this->cache_path.$classID."/";
      // delete dir
      $total_bytes+= $this->drop($path, true);
    }
    // return total deleted bytes
    return $total_bytes;
  }
  
  /**
   * Delete component template cache dir from disk method
   *
   * @param int Class_ID main class
   * @param int Class_ID class template id
   *
   * @return int total deleted bytes
   */
  public function dropClassTemplateCache ($class, $template_class) {
    // validate
    if (
      !(is_numeric($class) && $class > 0 ) ||
      !( (is_numeric($template_class) && $template_class > 0 ) || is_array($template_class) )
    ) {
      return false;
    }

    // return total deleted bytes
    return $this->dropClassCache($class);
  }
  
  /**
   * Delete message cache dir from disk method
   *
   * @param int Catalogue_ID
   * @param int Subdivision_ID
   * @param int Sub_Class_ID
   * @param int Class_ID
   * @param mixed Message_ID
   *
   * @return int total deleted bytes
   */
  public function dropMessageCache ($catalogue, $sub, $cc, $class, $message) {
    // validate
    if (
      !( is_numeric($class) && $class > 0 ) ||
      !( (is_numeric($message) && $message > 0 ) || is_array($message) )
    ) {
      return false;
    }
    // total deleted bytes
    $total_bytes = 0;
    foreach ( (array)$message AS $mess_id ) {
      // cache file path
      $path = $this->cache_path.$class."/".$mess_id."/";
      // delete dir
      $total_bytes+= $this->drop($path, true);
    }
    // return total deleted bytes
    return $total_bytes;
  }

  /**
   * Set nocache marks in component fields
   *
   * @param array component data
   *
   * @param int counted nocache block
   */   
  public function nocacheStore (&$data) {
    // parsed values with need sequence
    $this->nocacheStoreKeys = array("RecordTemplateFull");
    // call parent
    return parent::nocacheStore($data);
  }

}

?>