<?php
/* $Id: nc_component.class.php 3935 2010-08-25 10:59:29Z denis $ */

class nc_Component extends nc_Essence {
  
  protected $db;
  protected $_class_id, $_system_table_id;
  // все поля компонента и их количество
  protected $_fields, $_field_count;
  // массив полей, попадающих в запрос, и переменные, им соответствующие
  protected $_fields_query, $_fields_vars;
  protected $_joins;
  
  /**
   * Constructor function
   */  
  public function __construct ( $class_id = 0,  $system_table_id = 0) {
    // load parent constructor
    parent::__construct();
    
    // set essence
    $this->essence = "Class";
    
    // system superior object
    $nc_core = nc_Core::get_object();
    // system db object
    if ( is_object($nc_core->db) ) $this->db = $nc_core->db;
    
    $class_id  = intval($class_id );
    $system_table_id = intval($system_table_id);
    
    // загружаем конкретный компонент
    if ( $class_id || $system_table_id ) {
      $this->_class_id = $class_id;
      $this->_system_table_id = $system_table_id;
      // загружаем часть запроса и доп. данные
      $this->_make_query();
    }
    
  }
  
  public function get_fields_query (  ) {
    if ( empty($this->_fields_query) ) return "";
    
    return join(', ', $this->_fields_query);
  }
  
  
  public function get_fields_vars ( ) {
    if ( empty($this->_fields_vars) ) return "";
    
    return join(', ', $this->_fields_vars);
  }

  
  protected function _load_fields () {
    if ( empty($this->fields) ) {
      $this->_fields = $this->db->get_results("SELECT `Field_ID` as `id`, `Field_Name` as `name`, 
                                     `TypeOfData_ID` as `type`, `Format` as `format`,
                                     ".(!$this->_system_table_id ? "`DoSearch` AS `search`" : "1 AS `search`")."
                                     FROM `Field`
                                     WHERE ".($this->_system_table_id ? "
                                     `System_Table_ID` = '".$this->_system_table_id."'" : "
                                     `Class_ID` = '".$this->_class_id."'")."
                                     ORDER BY `Priority`", ARRAY_A);
                                     
      if ( $this->db->is_error ) throw new nc_Exception_DB_Error($this->db->last_query, $this->db->last_error);
      $this->_field_count = $this->db->num_rows;
    }
  }
  
  public function get_date_field () {
    $this->_load_fields();
    
    for ( $i = 0; $i < $this->_field_count; $i++ ) {
      if ( $this->_fields[$i]['type'] == 8 
           && (  $this->_fields[$i]['format'] == 'event' || $this->_fields[$i]['format'] == 'event_date' ) ) 
        return $this->_fields['name'];
    }
    
    return 0;
  }
  
  public function get_joins () {
    return $this->_joins;
  }
  
  protected function _make_query () {
    $nc_core = nc_Core::get_object();
    
    $this->_load_fields();
    
    $convert2txt = "";
    $joins = "";
    $multilist_fileds =array(); 
    

    if ( $this->_system_table_id == 3 ) {
      $this->_fields_query = array('a.`User_ID`', 'a.`PermissionGroup_ID`');
      $this->_fields_vars = array('$f_RowID', '$f_UserGroup');
    }
    else {
      $this->_fields_query = array('a.`Message_ID`', 'a.`User_ID`', 'a.`IP`', 'a.`UserAgent`',
                           'a.`LastUser_ID`', 'a.`LastIP`', 'a.`LastUserAgent`',
                           'a.`Priority`', 'a.`Parent_Message_ID`', 'sub.`Subdivision_ID`',
                           'CONCAT( \''.$nc_core->SUB_FOLDER.'\', sub.`Hidden_URL`)',
                           'cc.`Sub_Class_ID`', 'cc.`EnglishName`');
      $this->_fields_vars = array( '$f_RowID', '$f_UserID', '$f_IP', '$f_UserAgent',
                           '$f_LastUserID', '$f_LastIP', '$f_LastUserAgent',
                           '$f_Priority',  '$f_Parent_Message_ID', '$f_Subdivision_ID',
                           '$subLink', '$f_Sub_Class_ID', '$cc_keyword');
                           
      $this->_joins .= " LEFT JOIN `Subdivision` AS sub ON sub.`Subdivision_ID` = a.`Subdivision_ID` 
                         LEFT JOIN `Sub_Class` AS cc ON cc.`Sub_Class_ID` = a.`Sub_Class_ID`";

    }

    $this->_fields_query[] = 'a.`Checked`';
    $this->_fields_query[] = 'a.`Created`';
    $this->_fields_query[] = 'a.`Keyword`'; 
    $this->_fields_query[] = 'a.`LastUpdated` + 0 AS LastUpdated';

    $this->_fields_vars[] = '$f_Checked';
    $this->_fields_vars[] = '$f_Created';
    $this->_fields_vars[] = '$f_Keyword';
    $this->_fields_vars[] = '$f_LastUpdated';


  
    if ( !$this->_system_table_id && $nc_core->admin_mode && $nc_core->AUTHORIZE_BY !== 'User_ID' ){
      $this->_fields_query[] = "uAdminInterfaceAdd.`".$nc_core->AUTHORIZE_BY."` AS f_AdminInterface_user_add ";
      $this->_fields_query[] = "uAdminInterfaceChange.`".$nc_core->AUTHORIZE_BY."` AS f_AdminInterface_user_change ";

      $this->_fields_vars[] = '$f_AdminInterface_user_add';
      $this->_fields_vars[] = '$f_AdminInterface_user_change';

      $this->_joins .= " LEFT JOIN `User` AS uAdminInterfaceAdd ON a.`User_ID` = uAdminInterfaceAdd.`User_ID`
                         LEFT JOIN `User` AS uAdminInterfaceChange ON a.`LastUser_ID` = uAdminInterfaceChange.`User_ID` ";
  }


    
    for($i = 0; $i <  $this->_field_count; $i++) {
      $field = & $this->_fields[$i]; 
      // skip OpenID type
      if ( $field['type'] == 11 ) continue;
   
      switch ($field['type']) {
        // list field
        case 4:
          $this->_joins .= " LEFT JOIN `Classificator_".$field['format']."` AS tbl".$field['id']." ON a.`".$field['name']."` = tbl".$field['id'].".".$field['format']."_ID ";
  
          $this->_fields_query[] = "tbl".$field['id'].".".$field['format']."_Name AS tbl".$field['id']."name";
          $this->_fields_query[] = "tbl".$field['id'].".".$field['format']."_ID AS tbl".$field['id']."id";
          $this->_fields_query[] = "tbl".$field['id'].".`Value` AS tbl".$field['id']."value ";
        
          $this->_fields_vars[] = "\$f_".$field['name'];
          $this->_fields_vars[] = "\$f_".$field['name']."_id";
          $this->_fields_vars[] = "\$f_".$field['name']."_value";
          break;
        
        // date field
        case 8:
          $this->_fields_query[] = "a.".$field['name'];
          $this->_fields_vars[] = "\$f_".$field['name'];
  
          $this->_fields_query[] = "YEAR(a.`".$field['name']."`)";
          $this->_fields_query[] = "MONTH(a.`".$field['name']."`)";
          $this->_fields_query[] = "DAY(a.`".$field['name']."`)";
          $this->_fields_query[] = "HOUR(a.`".$field['name']."`)";
          $this->_fields_query[] = "MINUTE(a.`".$field['name']."`)";
          $this->_fields_query[] = "SECOND(a.`".$field['name']."`)";
  
          $this->_fields_vars[] = "\$f_".$field['name']."_year";
          $this->_fields_vars[] = "\$f_".$field['name']."_month";
          $this->_fields_vars[] = "\$f_".$field['name']."_day";
          $this->_fields_vars[] = "\$f_".$field['name']."_hours";
          $this->_fields_vars[] = "\$f_".$field['name']."_minutes";
          $this->_fields_vars[] = "\$f_".$field['name']."_seconds";
  
          
          break;
      
        // MultiList
        case 10:
          $multilist_fileds[] = array('name' => $field['name'], 'table' => strtok($field['format'], ':') );
          $this->_fields_query[] = "a.".$field['name'];
          $this->_fields_vars[] = "\$f_".$field['name'];
          break;
        
        default:
          $this->_fields_query[] = "a.".$field['name'];
          $this->_fields_vars[] = "\$f_".$field['name'];
          break;
      }
    }
  }
  
  public function get_fields ( $type = 0, $output_all = 1) {
    $this->_load_fields();
    
    $result = array();
    for ( $i = 0; $i < $this->_field_count; $i++ ) {
      if ( $this->_fields[$i]['type'] == $type ) {
        if ( $type == 10 ) $this->_fields[$i]['table'] = strtok($this->_fields[$i]['format'], ':');

        if ( $output_all ) {
          $result[] = $this->_fields[$i];
        }
        else {
          $result[$this->_fields[$i]['id']] = $this->_fields[$i]['name'];
        }

      }
    }
    
    return $result;
  }
  

  public function get_search_query ( $srchPat ) {
    // return if search params not setted
  if ( empty($srchPat) ) return array("query" => "", "link" => "");

  $search_param = array();
  
  for($i=0,$j=0; $i < $this->_field_count; $i++) { 
    $field = &$this->_fields[$i];

    if ($field['search'] ) {
      switch($field['type']) {
        case 1:   // Char
          if($srchPat[$j] == "") break;
          $fullSearchStr .= " AND a.".$field['name']." LIKE '%".urldecode($this->db->escape($srchPat[$j]))."%'";
          $search_param[] = "srchPat[".$j."]=".urldecode($srchPat[$j]);
          break;
        case 2:   // Int
          if($srchPat[$j] != "") {
             $srchPat[$j] += 0;
             $fullSearchStr .= " AND a.".$field['name'].">=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
          $j++;
          if($srchPat[$j] != "") {
             $srchPat[$j] += 0;
             $fullSearchStr .= " AND a.".$field['name']."<=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
          break;
        case 3:   // Text
          if($srchPat[$j] == "") break;
          $srch_str = $this->db->escape($srchPat[$j]);
          $fullSearchStr .= " AND a.".$field['name']." LIKE '%".urldecode($srch_str)."%'";
          $search_param[] = "srchPat[".$j."]=".urldecode($srchPat[$j]);
          break;
        case 4: // List
          if($srchPat[$j] == "") break;
          $srchPat[$j] += 0;
          $fullSearchStr .= " AND a.".$field['name']."=".$srchPat[$j];
          $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          break;
        case 5:   // Boolean
          if($srchPat[$j] == "") break;
          $srchPat[$j] += 0;
          $fullSearchStr .= " AND a.".$field['name']."=".$srchPat[$j];
          $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          break;
        case 6:   // File
          if($srchPat[$j] == "") break;
          $srch_str = $this->db->escape($srchPat[$j]);
          $fullSearchStr .= " AND SUBSTRING_INDEX(a.".$field['name'].",':',1) LIKE '%".urldecode($srch_str)."%'";
          $search_param[] = "srchPat[".$j."]=".urldecode($srchPat[$j]);
          break;
        case 7:   // Float
          if($srchPat[$j] != "") {
             $srchPat[$j] = floatval($srchPat[$j]);
             $fullSearchStr .= " AND a.".$field['name'].">=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
          $j++;
          if($srchPat[$j] != "") {
             $srchPat[$j] = floatval($srchPat[$j]);
             $fullSearchStr .= " AND a.".$field['name']."<=".$srchPat[$j];
             $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          }
        break;
        case 8:   // DateTime
          $date_from['d'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['m'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['Y'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%04d", $srchPat[$j]) : false); $j++;
          $date_from['H'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['i'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_from['s'] =($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['d'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['m'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['Y'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%04d", $srchPat[$j]) : false); $j++;
          $date_to['H'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['i'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false); $j++;
          $date_to['s'] = ($srchPat[$j] && ($search_param[] = "srchPat[".$j."]=".$srchPat[$j]) ? sprintf("%02d", $srchPat[$j]) : false);

          $date_format_from = ($date_from['Y'] ? '%Y' : '').($date_from['m'] ? '%m' : '').($date_from['d'] ? '%d' : '').($date_from['H'] ? '%H' : '').($date_from['i'] ? '%i' : '').($date_from['s'] ? '%s' : '');
          $date_format_to = ($date_to['Y'] ? '%Y' : '').($date_to['m'] ? '%m' : '').($date_to['d'] ? '%d' : '').($date_to['H'] ? '%H' : '').($date_to['i'] ? '%i' : '').($date_to['s'] ? '%s' : '');

          if($date_format_from) $fullSearchStr .= " AND DATE_FORMAT(a.".$field['name'].",'".$date_format_from."')>=".$date_from['Y'].$date_from['m'].$date_from['d'].$date_from['H'].$date_from['i'].$date_from['s'];
          if($date_format_to) $fullSearchStr .= " AND DATE_FORMAT(a.".$field['name'].",'".$date_format_to."')<=".$date_to['Y'].$date_to['m'].$date_to['d'].$date_to['H'].$date_to['i'].$date_to['s'];

          break;

          case 10: // MultiList
            if($srchPat[$j] == ""){ $j++; break; }

            $id = array(); // массив с id искомых элементов

            if (is_array($srchPat[$j])) {
              foreach ( (array)$srchPat[$j] as $v ) {
                $id[] = intval($v);
              }
            }
            else {
              $temp_id = explode('-', $srchPat[$j]);
              foreach ( (array)$temp_id as  $v ) {
                $id[] = intval($v);
              }
            }
            $j++; //второй параметр - это тип посика

            if ( empty($id) ) break;

            $fullSearchStr .= " AND (";
            switch ($srchPat[$j]) {
              case 1: //Полное совпадение
                $fullSearchStr .= "a.".$field['name']." LIKE CONCAT(',' ,  '".join(',', $id)."', ',') ";
                break;

              case 2: //Хотя бы один. Выбор между LIKE и REGEXP выпал в сторону первого
                foreach ( $id as $v ) $fullSearchStr .= "a.".$field['name']." LIKE CONCAT('%,', '".$v."', ',%') OR ";
                $fullSearchStr .= "0 "; //чтобы "закрыть" последний OR
                break;
             case 0: // как минимум выбранные - частичное совпадение - по умолчанию
              default:
                $srchPat[$j] = 0;
                $fullSearchStr .= "a.".$field['name']."  REGEXP  \"((,[0-9]+)*)";
                $prev_v =  -1;
                foreach ( $id as $v ) {
                  $fullSearchStr .= "(,".$v.",)([0-9]*)((,[0-9]+)*)";
                  $prev_v = $v;
                }
                $fullSearchStr .= '"';
                break;
            }
            $fullSearchStr .= ")";

            $search_param[] = "srchPat[".($j-1)."]=".join('-', $id);
            $search_param[] = "srchPat[".$j."]=".$srchPat[$j];
          break;
      }
      $j++;
    }
  }

  if ( !empty($search_param)) $search_params['link'] = join('&amp;', $search_param);
  $search_params['query'] = $fullSearchStr;

  return $search_params;
  }
}


?>
